"""
The Sanitize module is dedicated to adapters (stuff) that keeps
the model clean and in sync with diagrams.
"""

from zope import interface
from zope import component
from gaphor import UML
from gaphor.core import inject
from gaphor.interfaces import IService


class EventDispatcher(object):
    """
    Does some background cleanup jobs, such as removing elements from the
    model that have no presentations (and should have some).
    """
    interface.implements(IService)

    component_registry = inject('component_registry')


    def __init__(self):
        pass


    def init(self, app=None):
        pass


    def shutdown(self):
        pass
        

    def register_handler(self, factory, adapts=None):
        """
        Register a handler. Handlers are triggered (executed) when specific
        events are emitted through the handle() method.
        """
        self.component_registry.register_handler(factory, adapts)


    def unregister_handler(self, factory=None, required=None):
        """
        Unregister a previously registered handler.
        """
        self.component_registry.unregister_handler(factory, required)


    def fire(self, *events):
        """
        Send out one or more events.
        """
        self.component_registry.handle(events)


# vim:sw=4:et:ai



# vim:sw=4:et
"""
Feature diagram item. Feature is a super class of both Attribute, Operations and
Methods.
"""

from gaphor.diagram.compartment import FeatureItem
from gaphor.diagram import font


class AttributeItem(FeatureItem):

    def __init__(self, id=None):
        FeatureItem.__init__(self, id)

        self.watch('subject<NamedItem>.name') \
            .watch('subject<Property>.isDerived') \
            .watch('subject<Property>.visibility') \
            .watch('subject<Property>.isStatic') \
            .watch('subject<Property>.lowerValue<LiteralSpecification>.value') \
            .watch('subject<Property>.upperValue<LiteralSpecification>.value') \
            .watch('subject<Property>.defaultValue<LiteralSpecification>.value') \
            .watch('subject<Property>.typeValue<LiteralSpecification>.value')


    def postload(self):
        if self.subject:
            self.text = self.render()


class OperationItem(FeatureItem):

    def __init__(self, id=None):
        FeatureItem.__init__(self, id)
        
        self.watch('subject.name') \
            .watch('subject.isAbstract', self.on_operation_is_abstract) \
            .watch('subject.visibility') \
            .watch('subject.returnResult.lowerValue<LiteralSpecification>.value') \
            .watch('subject.returnResult.upperValue<LiteralSpecification>.value') \
            .watch('subject.returnResult.typeValue<LiteralSpecification>.value') \
            .watch('subject.formalParameter.lowerValue<LiteralSpecification>.value') \
            .watch('subject.formalParameter.upperValue<LiteralSpecification>.value') \
            .watch('subject.formalParameter.typeValue<LiteralSpecification>.value') \
            .watch('subject.formalParameter.defaultValue<LiteralSpecification>.value') \


    def postload(self):
        if self.subject:
            self.text = self.render()
        self.on_operation_is_abstract(None)

    def on_operation_is_abstract(self, event):
        self.font = (self.subject and self.subject.isAbstract) \
                and font.FONT_ABSTRACT_NAME or font.FONT_NAME
        self.request_update()

    def render(self):
        return self.subject.render(visibility=True, type=True, multiplicity=True, default=True) or ''


# vim:sw=4:et:ai



"""
The Sanitize module is dedicated to adapters (stuff) that keeps
the model clean and in sync with diagrams.
"""

from zope import interface
from zope import component
from gaphor import UML
from gaphor.UML.interfaces import IElementDeleteEvent
from gaphor.interfaces import IService


class SanitizerService(object):
    """
    Does some background cleanup jobs, such as removing elements from the
    model that have no presentations (and should have some).
    """
    interface.implements(IService)

    def __init__(self):
        pass

    def init(self, app):
        self._app = app
        app.register_handler(self._unlink_on_presentation_delete)

    def shutdown(self):
        self._app.unregister_handler(self._unlink_on_presentation_delete)
        
    @component.adapter(UML.Presentation, IElementDeleteEvent)
    def _unlink_on_presentation_delete(self, item, event):
        """
        Unlink the model element if no more presentations link to the `item`'s
        subject or the to-be-deleted item is the only item currently linked.
        """
        subject = item.subject
        if subject:
            presentation = subject.presentation
            if not presentation or \
                    (len(presentation) == 1 and presentation[0] is item):
                subject.unlink()


# vim:sw=4:et:ai
